/* ZBOSS Zigbee 3.0
 *
 * Copyright (c) 2012-2018 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software, with or without modification, must only be used with a Nordic
 *    Semiconductor ASA integrated circuit.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
PURPOSE: MAC definitions NRF52840 SoC.
*/

#ifndef __MAC_NRF52840_H
#define __MAC_NRF52840_H


#include "stddef.h"

#ifdef ZB_USE_DSR_RADIO_DRIVER
#include "mac_nrf52840_dsr_rd.h"
#endif

#include "mac_nrf52_transceiver.h"

#define ZB_TRANSCEIVER_START_CHANNEL_NUMBER 11
#define ZB_TRANSCEIVER_MAX_CHANNEL_NUMBER   26

void mac_nrf52840_hw_init(void);
#define ZB_TRANSCEIVER_INIT_RADIO()
#define ZB_TRANSCEIVER_DEINIT_RADIO()

void mac_nrf52840_abort_tx(void);
#define ZB_MAC_TX_ABORT() mac_nrf52840_abort_tx()

#define ZB_RADIO_INT_DISABLE() ZB_OSIF_GLOBAL_LOCK()
#define ZB_RADIO_INT_ENABLE()  ZB_OSIF_GLOBAL_UNLOCK()

#define ZB_MAC_SET_TRANS_INT() (TRANS_CTX().interrupt_flag = 1)
#define ZB_MAC_CLEAR_TRANS_INT() (TRANS_CTX().interrupt_flag = 0)
#define ZB_MAC_GET_TRANS_INT_FLAG() (TRANS_CTX().interrupt_flag)

#define ZB_MAC_READ_INT_STATUS_REG() /* already done by int handler */

#define ZB_MAC_GET_RX_INT_STATUS_BIT() TRANS_CTX().rx_int
#define ZB_MAC_GET_TX_INT_STATUS_BIT() TRANS_CTX().tx_int

#define ZB_MAC_SET_RX_INT_STATUS_BIT() TRANS_CTX().rx_int = 1
#define ZB_MAC_SET_TX_INT_STATUS_BIT() TRANS_CTX().tx_int = 1

#define ZB_MAC_CLEAR_RX_INT_STATUS_BIT() TRANS_CTX().rx_int = 0
#define ZB_MAC_CLEAR_TX_INT_STATUS_BIT() (TRANS_CTX().tx_int = TRANS_CTX().tx_status = 0)

#define ZB_TRANS_CHECK_CHANNEL_BUSY_ERROR() (TRANS_CTX().tx_status == ZB_TRANS_CHANNEL_BUSY_ERROR)
#define ZB_TRANS_CHECK_TX_RETRY_COUNT_EXCEEDED_ERROR() 0
#define ZB_TRANS_CHECK_NO_ACK() (TRANS_CTX().tx_status == ZB_TRANS_NO_ACK)

#define ZB_TRANSCEIVER_SET_COORD_SHORT_ADDR(addr)
#define ZB_TRANSCEIVER_SET_COORD_EXT_ADDR(addr)

void zb_transceiver_set_pan_coord(zb_bool_t pan_coord);
#define ZB_TRANSCEIVER_SET_PAN_COORDINATOR(pan_coord) zb_transceiver_set_pan_coord((zb_bool_t)pan_coord)

#ifdef ZB_MAC_SOFTWARE_PB_MATCHING

zb_bool_t mac_nrf52840_pending_bit(void);
#define ZB_MAC_TRANS_PENDING_BIT()        mac_nrf52840_pending_bit()

#define ZB_MAC_TRANS_CLEAR_PENDING_BIT()
#define ZB_MAC_TRANS_SET_PENDING_BIT()

#elif (defined ZB_MAC_STICKY_PENDING_BIT) /* #if !ZB_MAC_SOFTWARE_PB_MATCHING */

zb_bool_t mac_nrf52840_pending_bit(void);
#define ZB_MAC_TRANS_PENDING_BIT()        mac_nrf52840_pending_bit()

/* FIXME: implement for sticky if needed */
#define ZB_MAC_TRANS_CLEAR_PENDING_BIT()
// nrf_802154_pending_bit_for_addr_clear()
#define ZB_MAC_TRANS_SET_PENDING_BIT()
// nrf_802154_pending_bit_for_addr_set()

#elif (defined ZB_MAC_HARDWARE_PB_MATCHING)

/* MAC source matching logic implemented in radio-driver v1.6.0:
 * 1) Add device address in the driver's pending table if there is no indirect data for it (clear pending bit);
 * 2) Remove device address from he driver's pending table if there is an indirect data for it (set pending bit);
*/

zb_uint8_t mac_nrf52840_src_match_set_pending_bit(zb_uint8_t *addr, zb_bool_t value, zb_bool_t extended);
void zb_transceiver_src_match_tbl_drop(void);


#define ZB_TRANSCEIVER_SRC_MATCH_SHORT_SET_PENDING_BIT(short_addr, value) \
  mac_nrf52840_src_match_set_pending_bit((zb_uint8_t *)&(short_addr), (zb_bool_t)(value), ZB_FALSE)

#define ZB_TRANSCEIVER_SRC_MATCH_ADD_SHORT_ADDR(index, short_addr) \
  mac_nrf52840_src_match_set_pending_bit((zb_uint8_t *)&(short_addr), ZB_FALSE, ZB_FALSE)

#define ZB_TRANSCEIVER_SRC_MATCH_DELETE_SHORT_ADDR(index, short_addr) \
  mac_nrf52840_src_match_set_pending_bit((zb_uint8_t *)&(short_addr), ZB_TRUE, ZB_FALSE)

#define ZB_TRANSCEIVER_SRC_MATCH_IEEE_SET_PENDING_BIT(ieee_addr, value) \
  mac_nrf52840_src_match_set_pending_bit((zb_uint8_t *)(ieee_addr), (zb_bool_t)(value), ZB_TRUE)

#define ZB_TRANSCEIVER_SRC_MATCH_ADD_IEEE_ADDR(index, ieee_addr) \
  mac_nrf52840_src_match_set_pending_bit((zb_uint8_t *)(ieee_addr), ZB_FALSE, ZB_TRUE)

#define ZB_TRANSCEIVER_SRC_MATCH_DELETE_IEEE_ADDR(index, ieee_addr) \
  mac_nrf52840_src_match_set_pending_bit((zb_uint8_t *)(ieee_addr), ZB_TRUE, ZB_TRUE)

#define ZB_TRANSCEIVER_SRC_MATCH_TBL_DROP() \
  zb_transceiver_src_match_tbl_drop()

/* Don't send the empty data frame afrer low-level RX. Deside what to do later, during the DATA.Request processing. */
#define ZB_MAC_TRANS_PENDING_BIT() ZB_FALSE

/* N/A */
#define ZB_MAC_TRANS_CLEAR_PENDING_BIT()
#define ZB_MAC_TRANS_SET_PENDING_BIT()

#endif  /* !ZB_MAC_HARDWARE_PB_MATCHING */


void zb_transceiver_set_short_addr(zb_uint16_t addr);
#define ZB_TRANSCEIVER_UPDATE_SHORT_ADDR() zb_transceiver_set_short_addr(MAC_PIB().mac_short_address)

void zb_transceiver_set_pan_id(zb_uint16_t pan_id);
#define ZB_TRANSCEIVER_SET_PAN_ID(pan_id) zb_transceiver_set_pan_id(pan_id)

#define ZB_TRANSCEIVER_UPDATE_PAN_ID() ZB_TRANSCEIVER_SET_PAN_ID(MAC_PIB().mac_pan_id)

#define ZB_RADIO_INT_DISABLE() ZB_OSIF_GLOBAL_LOCK()
#define ZB_RADIO_INT_ENABLE()  ZB_OSIF_GLOBAL_UNLOCK()

/* According to BZ#14383 and J#ZB-72 this workaround is not needed, if it is needed to put
 * transceiver in idle it should be done automatically in the MAC logic. */
#define ZB_MAC_TRANSCEIVER_GO_IDLE() /* not needed */
#define ZB_MAC_TRANSCEIVER_RESTORE() /* not needed */

#define ZB_TRANSCEIVER_SET_PROMISCUOUS(promiscuous_mode)

void mac_nrf52840_update_long_mac(void);
#define ZB_TRANSCEIVER_UPDATE_LONGMAC()         \
  mac_nrf52840_update_long_mac()

void zb_transceiver_get_rssi(zb_uint8_t *rssi_value);
#define ZB_TRANSCEIVER_GET_RSSI zb_transceiver_get_rssi

#define ZB_TRANSCEIVER_GET_ENERGY_LEVEL ZB_TRANSCEIVER_GET_RSSI

void mac_nrf52840_start_get_rssi(zb_uint8_t scan_duration_bi);
#define ZB_TRANSCEIVER_START_GET_RSSI(_scan_duration_bi) mac_nrf52840_start_get_rssi(_scan_duration_bi);

void zb_transceiver_set_channel(zb_uint8_t channel_number);
#define ZB_TRANSCEIVER_SET_CHANNEL(page, channel_number) zb_transceiver_set_channel(channel_number)

void mac_nrf52840_send_packet(zb_bufid_t buf, zb_uint8_t wait_type);

#define ZB_TRANS_SEND_FRAME(header_length, buf, wait_type)      \
  {                                                             \
    ZB_DUMP_OUTGOING_DATA(buf);                                 \
    (void)header_length;                                        \
    mac_nrf52840_send_packet((buf), (wait_type));               \
  }

void mac_nrf52840_resend_packet(zb_uint8_t wait_type, zb_time_t tx_at);

#define ZB_TRANS_REPEAT_SEND_FRAME(header_length, buf, wait_type)       \
  (ZB_DUMP_OUTGOING_DATA(buf),                                          \
   (void)header_length, mac_nrf52840_resend_packet(wait_type, 0))

void mac_nrf52840_set_accept_ack(zb_uint_t accept_ack);
#define ZB_TRANSCEIVER_ACK_ACCEPT(enable_ack) mac_nrf52840_set_accept_ack(enable_ack)

void mac_nrf52840_trans_set_rx_on_off(int rx_on);
zb_bool_t zb_transceiver_get_rx_on_off(void);
zb_bool_t zb_transceiver_get_radio_on_off(void);

#define ZB_TRANSCEIVER_SET_RX_ON_OFF(_rx_on) mac_nrf52840_trans_set_rx_on_off(_rx_on)
#define ZB_TRANSCEIVER_GET_RX_ON_OFF()       zb_transceiver_get_rx_on_off()
#define ZB_TRANSCEIVER_RADIO_ON_OFF()        zb_transceiver_get_radio_on_off()

#define MAC_TIMER_QUANT_US 4
#define ZB_TRANS_GET_TX_TIMESTAMP()          osif_transceiver_time_get()

/*PHY tests specific API. Implement for PHY tests only.*/
#define ZB_TRANSCEIVER_PERFORM_CCA()         (0) /*Perform single CCA attempt.
                                                   Used in TP_154_PHY24_RECEIVER_07 test*/
#define ZB_TRANS_TX_CARRIER_DATA(channel, timeout_bi) (0) /*Start continuous transmission.
                                                            Used in TP_154_PHY24_TRANSMIT_02 test.*/

void zb_transceiver_set_auto_ack(zb_bool_t auto_ack);
#define ZB_TRANSCEIVER_ENABLE_AUTO_ACK() zb_transceiver_set_auto_ack(ZB_TRUE)

void zb_transceiver_set_tx_power(zb_int8_t power);
#define ZB_TRANSCEIVER_SET_TX_POWER(new_power) zb_transceiver_set_tx_power(new_power)

void zb_transceiver_get_tx_power(zb_int8_t* power);
#define ZB_TRANSCEIVER_GET_TX_POWER(power) zb_transceiver_get_tx_power(power)


/**NRF52840 radio specific API*/

/**
   Cut transceiver-specific header (length byte)
*/
#define ZB_TRANS_CUT_SPECIFIC_HEADER(zb_buffer) /*(void)zb_buf_cut_left((zb_buffer), 1);*/

#define ZB_RX_FLUSH_TIMEOUT (60 * ZB_TIME_ONE_SECOND)
#define NRF52840_CCA_LENGTH 0x34 /*default measurement time for CCA equal to 208�s*/

#define NRF52840_TX_STARTED 1
#define NRF52840_TX_STOPPED 0

#define NRF52840_ED_STOPPED 0
#define NRF52840_ED_STARTED 1
#define NRF52840_ED_ERROR 2

#define NRF52840_LEN_SIZE 1
#define NRF52840_CRC_SIZE 2
#define NRF52840_MAX_RETRANSMIT_ATTEMPTS 4

#define NRF52840_PAN_ID_SIZE           2    // Size of Pan ID
#define NRF52840_SHORT_ADDRESS_SIZE    2    // Size of Short Mac Address
#define NRF52840_EXTENDED_ADDRESS_SIZE 8    // Size of Extended Mac Address
#define NRF52840_MAX_PACKET_SIZE       127  // Maximal size of radio packet
#define NRF52840_PHR_SIZE              1    // Length field size of radio TX buffer
#define NRF52840_ACK_LENGTH            5    // MAC ack length

#define NRF52840_CRC_LENGTH      2         // Length of CRC in 802.15.4 frames [bytes]
#define NRF52840_CRC_POLYNOMIAL  0x011021  // Polynomial used for CRC calculation in 802.15.4 frames
#define NRF52840_MHMU_MASK               0xff0007ff // Mask of known bytes in ACK packet
#define NRF52840_MHMU_PATTERN            0x00000205 // Values of known bytes in ACK packet

#define NRF52840_ACK_HEADER_WITH_PENDING      0x12  // First byte of ACK frame containing pending bit
#define NRF52840_ACK_HEADER_WITHOUT_PENDING   0x02  // First byte of ACK frame without pending bit

#define NRF52840_ACK_REQUEST_BIT              (1 << 5)  // Ack request bit
#define NRF52840_ACK_REQUEST_OFFSET           1         // Byte containing Ack request bit (+1 for frame length byte)
#define NRF52840_DEST_ADDR_TYPE_EXTENDED      0x0c      // Bits containing extended destination address type
#define NRF52840_DEST_ADDR_TYPE_MASK          0x0c      // Mask of bits containing destination address type
#define NRF52840_DEST_ADDR_TYPE_OFFSET        2         // Byte containing destination address type (+1 for frame length byte)
#define NRF52840_DEST_ADDR_TYPE_SHORT         0x08      // Bits containing short destination address type
#define NRF52840_DEST_ADDR_OFFSET             5         // Offset of destination address in Data frame
#define NRF52840_DSN_OFFSET                   3         // Byte containing DSN value (+1 for frame length byte)
#define NRF52840_FRAME_PENDING_BIT            (1 << 4)  // Pending bit
#define NRF52840_FRAME_PENDING_OFFSET         1         // Byte containing pending bit (+1 for frame length byte)
#define NRF52840_FRAME_TYPE_ACK               0x02      // Bits containing ACK frame type
#define NRF52840_ACK_FRAME_LEN_OFFSET         0x0       // ACK frame length offset
#define NRF52840_ACK_FRAME_DSN_OFFSET         0x3       // ACK frame DSN offset
#define NRF52840_ACK_FRAME_LEN                0x05      // ACK frame length
#define NRF52840_FRAME_TYPE_BEACON            0x00      // Bits containing Beacon frame type
#define NRF52840_FRAME_TYPE_COMMAND           0x03      // Bits containing Command frame type
#define NRF52840_FRAME_TYPE_DATA              0x01      // Bits containing Data frame type
#define NRF52840_FRAME_TYPE_MASK              0x07      // Mask of bits containing frame type
#define NRF52840_FRAME_TYPE_OFFSET            1         // Byte containing frame type bits (+1 for frame length byte)
#define NRF52840_PAN_ID_COMPR_OFFSET          1         // Byte containing Pan ID compression bit (+1 for frame length byte)
#define NRF52840_PAN_ID_COMPR_MASK            0x40      // Pan ID compression bit
#define NRF52840_PAN_ID_OFFSET                3         // Offset of Pan ID in Data frame
#define NRF52840_SRC_ADDR_TYPE_EXTENDED       0xc0      // Bits containing extended source address type
#define NRF52840_SRC_ADDR_TYPE_MASK           0xc0      // Mask of bits containing source address type
#define NRF52840_SRC_ADDR_TYPE_OFFSET         2         // Byte containing source address type (+1 for frame length byte)
#define NRF52840_SRC_ADDR_TYPE_SHORT          0x80      // Bits containing short source address type
#define NRF52840_SRC_ADDR_OFFSET_SHORT_DST    8         // Offset of source address in Data frame if destination address is short
#define NRF52840_SRC_ADDR_OFFSET_EXTENDED_DST 14        // Offset of source address in Data frame if destination address is extended

typedef struct zb_transceiver_ctx_s
{
  zb_uint_t tx_int;
  zb_uint_t rx_int;
  zb_uint_t tx_in_progress;
  zb_uint_t tx_status;
  zb_uint_t pending_bit;
  /* 5/11/2017 EE CR:MINOR remove not used fields */
  zb_uint8_t interrupt_flag;
  zb_uint8_t recv_buf_full;
  zb_uint8_t ed_started;
#ifdef ZB_USE_DSR_RADIO_DRIVER
  zb_uint8_t m_pan_id[NRF52840_PAN_ID_SIZE];                             // Pan ID of this node
  zb_uint8_t m_short_addr[NRF52840_SHORT_ADDRESS_SIZE];                  // Short Address of this node
  zb_uint8_t m_extended_addr[NRF52840_EXTENDED_ADDRESS_SIZE];            // Extended Address of this node
  radio_state_t m_state;                                                 // State of the radio driver
#endif /*#ifdef ZB_USE_DSR_RADIO_DRIVER*/
  zb_uint8_t m_tx_buf[NRF52840_MAX_PACKET_SIZE + NRF52840_PHR_SIZE];     // NRF TX buffer lead by the packet length byte
  zb_uint8_t m_rx_buf[NRF52840_MAX_PACKET_SIZE + NRF52840_PHR_SIZE];     // NRF RX buffer
#ifdef ZB_USE_DSR_RADIO_DRIVER
  zb_uint8_t m_ack_buf[NRF52840_ACK_LENGTH + NRF52840_PHR_SIZE];         // NRF ACK buffer
  zb_uint8_t ack_waiting;
  zb_uint8_t csma_backoffs;
#endif
  zb_uint8_t rssi;
  zb_uint8_t zgpfs_count;
  zb_time_t  tx_timestamp;
  zb_time_t  rx_timestamp;
  zb_uint_t  failed_tx;
  zb_uint8_t inited;
}
zb_transceiver_ctx_t;

#endif /* __MAC_NRF52840_H */
